/*
Module : ChooseDirDlg.cpp
Purpose: Implementation for an MFC UI class to get a directory/folder using the Shell API SHBrowseForFolder
Created: PJN / 26-11-1997
History: PJN / 03-04-1999 General code clean-up. 
         PJN / 19-07-2006 1. Updated copyright details.
                          2. Addition of a CCHOOSEDIRDLG_EXT_CLASS preprocessor macro to allow the code to be
                          easily added to an extension dll.
         PJN / 18-12-2008 1. Updated copyright details.
                          2. Removed VC 6 style AppWizard comments from the code
                          3. The code has now been updated to support VC 2005 or later only
                          4. Code now compiles cleanly using Code Analysis (/analyze)
                          5. Refactored the CChooseDirDlg codebase quite a bit
         PJN / 23-09-2017 1. Reworked the CChooseDirDlg class to use IFileDialog interface.
         PJN / 08-07-2018 1. Updated copyright details.
                          2. Fixed a number of C++ core guidelines compiler warnings. These changes mean that
                          the code will now only compile on VC 2017 or later.
         PJN / 20-12-2019 1. Fixed various Clang-Tidy static code analysis warnings in the code.
         PJN / 14-03-2020 1. Updated copyright details.
                          2. Fixed more Clang-Tidy static code analysis warnings in the code.

Copyright (c) 1997 - 2022 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/


//////////////////// Includes /////////////////////////////////////////////////

#include "stdafx.h"
#include "ChooseDirDlg.h"


//////////////////// Defines //////////////////////////////////////////////////

#ifdef _DEBUG
#define new DEBUG_NEW
#endif //#ifdef _DEBUG


//////////////////// Implementation ///////////////////////////////////////////

CChooseDirDlg::CChooseDirDlg(_In_ CWnd* pParentWnd, _In_ const CString& sTitle, _In_ FILEOPENDIALOGOPTIONS dwFlags) : m_dwFlags{dwFlags},
                                                                                                                      m_sTitle{sTitle},
                                                                                                                      m_pParentWnd{pParentWnd}
{
}

HRESULT CChooseDirDlg::DoModal(_Inout_ CString& sDir)
{
  HRESULT hr{CoCreateInstance(CLSID_FileOpenDialog, nullptr, CLSCTX_INPROC_SERVER)};
  if (FAILED(hr))
  {
    TRACE(_T("CChooseDirDlg::DoModal, Failed to create IFileDialog interface, Error:%08X\n"), hr);
    return hr;
  }

  //Set the options for the dialog
  IFileDialog* pFileDialog{operator->()};
#pragma warning(suppress: 26496)
  AFXASSUME(pFileDialog != nullptr);
  DWORD dwDefaultFlags{0};
  hr = pFileDialog->GetOptions(&dwDefaultFlags);
  if (FAILED(hr))
  {
    TRACE(_T("CChooseDirDlg::DoModal, Failed to call IFileDialog::GetOptions, Error:%08X\n"), hr);
    return hr;
  }
  hr = pFileDialog->SetOptions(dwDefaultFlags | m_dwFlags);
  if (FAILED(hr))
  {
    TRACE(_T("CChooseDirDlg::DoModal, Failed to call IFileDialog::SetOptions, Error:%08X\n"), hr);
    return hr;
  }
  hr = pFileDialog->SetTitle(CStringW{m_sTitle});
  if (FAILED(hr))
  {
    TRACE(_T("CChooseDirDlg::DoModal, Failed to call IFileDialog::SetTitle, Error:%08X\n"), hr);
    return hr;
  }

  //Set the initial folder
  hr = pFileDialog->SetFileName(CStringW{sDir});
  if (FAILED(hr))
  {
    TRACE(_T("CChooseDirDlg::DoModal, Failed to call IFileDialog::SetFileName, Error:%08X\n"), hr);
    return hr;
  }

  //Show the dialog
  hr = pFileDialog->Show(m_pParentWnd->GetSafeHwnd());
  if (FAILED(hr))
  {
    TRACE(_T("CChooseDirDlg::DoModal, Failed to call IFileDialog::Show, Error:%08X\n"), hr);
    return hr;
  }

  //Get the new folder value
  ATL::CComPtr<IShellItem> si;
  hr = pFileDialog->GetResult(&si);
  if (FAILED(hr))
  {
    TRACE(_T("CChooseDirDlg::DoModal, Failed to call IFileDialog::GetResult, Error:%08X\n"), hr);
    return hr;
  }
  LPWSTR pszPath{nullptr};
  hr = si->GetDisplayName(SIGDN_FILESYSPATH, &pszPath);
  if (FAILED(hr))
  {
    TRACE(_T("CChooseDirDlg::DoModal, Failed to call IShellItem::GetDisplayName, Error:%08X\n"), hr);
    return hr;
  }

  //Update the output parameter and free the returned memory
  sDir = pszPath;
  CoTaskMemFree(pszPath);

  return hr;
}
